'use strict';

var mixin = require('node-mixin');

var net = require('net');
var http = require('http');
var url = require('url');
var http2 = require('./node-http2');
var removeDeprecatedHeaders = require('./header').removeDeprecatedHeaders;

function createHTTPServer(remoteSock, httpServerID, port) {
  var server = http.createServer(function(req, res) {
    var u = url.parse(req.url);
    var headers = removeDeprecatedHeaders(req.headers);

    headers['x-real-ip'] = req.connection.remoteAddress;

    var pReq = http2.raw.request({
      id      : httpServerID,
      plain   : true,
      socket  : remoteSock,
      port    : u.port || 80,
      path    : u.path,       
      method  : req.method,
      headers : headers
    }, function(pRes) {
      res.writeHead(pRes.statusCode, pRes.headers);
      pRes.pipe(res);
    });

    pReq.on('error', function(e) {
      res.writeHead(200);
      res.write('something was wrong.');
      res.end();
      return;
    });

    req.pipe(pReq);
  });

  server.listen(port, '0.0.0.0', function() {
    console.log('HTTPServer is running at port', port, '...');
  });

  return server;
}

function createTCPServer(options) {
  var httpServerID = 0;
  options = mixin(options,{port: 10000, httpConnects: 99});

  var tcpPort = options.port;
  var count = options.httpConnects;

  var ports = [];
  for(var i = 0; i < count; i++){
    ports.push(tcpPort + i + 1);
  }

  var httpServer;

  var server = net.createServer(function(sock) {
    var port = ports.shift();

    sock.on('data', function(data){
      data = data.toString('utf-8');
      if(data && data === 'port?'){
        sock.write('port:' + port); 
      }else if(data && data === 'start server'){
        httpServer = createHTTPServer(sock, httpServerID++, port);
        sock.on('close', function() {
          httpServer && httpServer.close();
          ports.unshift(port);
          console.log('Client disconnected!');
        });
        console.log('Client connected...');
      }
    });
  });

  server.listen(tcpPort, '0.0.0.0', function() {
    console.log('TCPServer is running at port', tcpPort, '...');
  });

  server.on('error', function(e) {
    console.log('Can not start server! ' + e.errno);
  });
}

module.exports = createTCPServer;